"""
Contains the config class (pyuic.cfg or pyuic.json)

"""
import json


class QtApi:
    pyqt4 = 0
    pyqt5 = 1
    pyside = 2


class Config:
    def __init__(self):
        self.files = []
        self.pyuic = ''
        self.pyuic_options = ''
        self.pyrcc = ''
        self.pyrcc_options = ''
        self.hooks = []

    def uic_command(self):
        return self.pyuic + ' ' + self.pyuic_options + ' %s -o %s'

    def rcc_command(self):
        return self.pyrcc + ' ' + self.pyrcc_options + ' %s -o %s'

    def load(self):
        for ext in ['.cfg', '.json']:
            try:
                with open('pyuic' + ext, 'r') as f:
                    self.__dict__ = json.load(f)
            except (IOError, OSError):
                pass
            else:
                break
        else:
            print('failed to open configuration file')
        if not hasattr(self, 'hooks'):
            self.hooks = []

    def save(self):
        with open('pyuic.json', 'w') as f:
            json.dump(self.__dict__, f, indent=4, sort_keys=True)

    def generate(self, api):
        if api == QtApi.pyqt4:
            self.pyrcc = 'pyrcc4'
            self.pyrcc_options = '-py3'
            self.pyuic = 'pyuic4'
            self.pyuic_options = '--from-import'
            self.files[:] = []
        elif api == QtApi.pyqt5:
            self.pyrcc = 'pyrcc5'
            self.pyrcc_options = ''
            self.pyuic = 'pyuic5'
            self.pyuic_options = '--from-import'
            self.files[:] = []
        elif api == QtApi.pyside:
            self.pyrcc = 'pyside-rcc'
            self.pyrcc_options = '-py3'
            self.pyuic = 'pyside-uic'
            self.pyuic_options = '--from-import'
            self.files[:] = []
        self.save()
        print('pyuic.json generated')

    def add(self, src, dst):
        self.load()
        for fn, _ in self.files:
            if fn == src:
                print('ui file already added: %s' % src)
                return
        self.files.append((src, dst))
        self.save()
        print('file added to pyuic.json: %s -> %s' % (src, dst))

    def remove(self, filename):
        self.load()
        to_remove = None
        for i, files in enumerate(self.files):
            src, dest = files
            if filename == src:
                to_remove = i
                break
        if to_remove is not None:
            self.files.pop(to_remove)
        self.save()
        print('file removed from pyuic.json: %s' % filename)



"""Help you manage your pyuic.json file (pyqt-distutils)

Usage:
    pyuicfg -g
    pyuicfg -g --pyqt5
    pyuicfg -g --pyside
    pyuicfg -a SOURCE_FILE DESTINATION_PACKAGE
    pyuicfg -r SOURCE_FILE
    pyuicfg (-h | --help)
    pyuicfg --version

Options:
    -h, --help                            Show help
    --version                             Show version
    -g                                    Generate pyuic.json
    -a SOURCE_FILE DESTINATION_PACKAGE    Add file to pyuic.json
    -r SOURCE_FILE                        Remove file from pyuic.json
    --pyqt5                               Generate a pyuic.json file for PyQt5 instead of PyQt4
    --pyside                              Generate a pyuic.json file for PySide instead of PyQt4

"""
import os
from docopt import docopt
from pyqt_distutils import __version__
from pyqt_distutils.config import Config, QtApi


def qt_api_from_args(arguments):
    if arguments['--pyqt5']:
        return QtApi.pyqt5
    elif arguments['--pyside']:
        return QtApi.pyside
    return QtApi.pyqt4


def main():
    arguments = docopt(__doc__, version=__version__)
    generate = arguments['-g']
    file_to_add = arguments['-a']
    destination_package = arguments['DESTINATION_PACKAGE']
    file_to_remove = arguments['-r']
    api = qt_api_from_args(arguments)
    cfg = Config()
    if generate:
        if os.path.exists('pyuic.json'):
            choice = input('pyuic.json already exists. Do you want to replace '
                           'it? (y/N) ').lower()
            if choice != 'y':
                return
        cfg.generate(api)
    elif file_to_add:
        cfg.add(file_to_add, destination_package)
    elif file_to_remove:
        cfg.remove(file_to_remove)


if __name__ == '__main__':
    main()


#!/usr/bin/env python
from setuptools import setup, find_packages

from pyqt_distutils import __version__


setup(
    name='pyqt-distutils',
    version=__version__,
    packages=find_packages(),
    url='https://github.com/ColinDuquesnoy/pyqt_distutils',
    license='MIT',
    author='Colin Duquesnoy',
    author_email='colin.duquesnoy@gmail.com',
    description='A set of distutils extension for building Qt ui files',
    install_requires=['docopt'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Setuptools Plugin',
        'Environment :: Console',
        'Environment :: X11 Applications :: Qt',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Microsoft',
        'Operating System :: MacOS',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2',
        'Topic :: Software Development :: Build Tools',
        'Topic :: System :: Software Distribution',
        'Topic :: Text Editors :: Integrated Development Environments (IDE)'
    ],
    entry_points={
        'console_scripts': ['pyuicfg = pyqt_distutils.pyuicfg:main'],
        'pyqt_distutils_hooks': ['gettext = pyqt_distutils.hooks:gettext']
    }
)



# -*- coding: utf-8 -*-
"""
Distutils extension for PyQt/PySide applications
"""
import glob
import os
from setuptools import Command

from .config import Config
from .hooks import load_hooks

class build_ui(Command):
    """
    Builds the Qt ui files as described in pyuic.json (or pyuic.cfg).
    """
    user_options = [
        ('force', None,
         'Force flag, will force recompilation of every ui/qrc file'),
    ]

    def initialize_options(self):
        self.force = False
        self._hooks = load_hooks()

    def finalize_options(self):
        try:
            self.cfg = Config()
            self.cfg.load()
        except (IOError, OSError):
            print('cannot open pyuic.json (or pyuic.cfg)')
            self.cfg = None

    def is_outdated(self, src, dst, ui):
        if src.endswith('.qrc') or self.force:
            return True
        outdated = (not os.path.exists(dst) or
                    (os.path.getmtime(src) > os.path.getmtime(dst)))
        if not outdated and not ui:
            # for qrc files, we need to check each individual resources.
            # If one of them is newer than the dst file, the qrc file must be
            # considered as outdated
            # file paths are relative to the qrc file path
            qrc_dirname = os.path.dirname(src)
            with open(src, 'r') as f:
                lines = f.read().splitlines()
                lines = [l for l in lines if '<file>' in l]
            cwd = os.getcwd()
            os.chdir(qrc_dirname)
            for line in lines:
                filename = line.replace('<file>', '').replace(
                    '</file>', '').strip()
                filename = os.path.abspath(filename)
                if os.path.getmtime(filename) > os.path.getmtime(dst):
                    outdated = True
                    break
            os.chdir(cwd)
        return outdated

    def run(self):
        if not self.cfg:
            return
        for glob_exp, dest in self.cfg.files:
            for src in glob.glob(glob_exp):
                if not os.path.exists(src):
                    print('skipping target %s, file not found' % src)
                    continue
                src = os.path.join(os.getcwd(), src)
                dst = os.path.join(os.getcwd(), dest)
                ui = True
                if src.endswith('.ui'):
                    ext = '_ui.py'
                    cmd = self.cfg.uic_command()
                elif src.endswith('.qrc'):
                    ui = False
                    ext = '_rc.py'
                    cmd = self.cfg.rcc_command()
                else:
                    continue
                filename = os.path.split(src)[1]
                filename = os.path.splitext(filename)[0]
                dst = os.path.join(dst, filename + ext)
                cmd = cmd % (src, dst)
                try:
                    os.makedirs(os.path.split(dst)[0])
                except OSError:
                    pass

                if self.is_outdated(src, dst, ui):
                    print(cmd)
                    os.system(cmd)
                    for hookname in self.cfg.hooks:
                        try:
                            hook = self._hooks[hookname]
                        except KeyError:
                            print('warning, unknonw hook: %r' % hookname)
                        else:
                            print('running hook %r' % hookname)
                            hook(dst)
                else:
                    print('skipping %s, up to date' % src)



"""
This module contains the hooks load and our builtin hooks.

"""
import re
import pkg_resources
import traceback


#: Name of the entrypoint to use in setup.py
ENTRYPOINT = 'pyqt_distutils_hooks'


def load_hooks():
    """
    Load the exposed hooks.

    Returns a dict of hooks where the keys are the name of the hook and the
    values are the actual hook functions.
    """
    hooks = {}
    for entrypoint in pkg_resources.iter_entry_points(ENTRYPOINT):
        name = str(entrypoint).split('=')[0].strip()
        try:
            hook = entrypoint.load()
        except Exception:
            traceback.print_exc()
        else:
            hooks[name] = hook
    return hooks


def hook(ui_file_path):
    """
    This is the prototype of a hook function.
    """
    pass


GETTEXT_REPLACEMENT = '''        import gettext
        _ = gettext.gettext
'''


def gettext(ui_file_path):
    """
    Let you use gettext instead of the Qt tools for l18n
    """
    with open(ui_file_path, 'r') as fin:
        content = fin.read()

    content = re.sub(r'_translate\(".*",\s', '_(', content)

    with open(ui_file_path, 'w') as fout:
        fout.write(content.replace(
            '        _translate = QtCore.QCoreApplication.translate',
            GETTEXT_REPLACEMENT))
